library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_signed.all;  

use work.MAC;             
use work.DIFF_nB;
use work.FIFO;
use work.Partie_Commande;


-- déclaration de l'entité
entity Filtre_FX_LMS is 
	generic(NB_BITS  : integer :=16 ;
			  NB_COEFF : integer :=50 ;
			  Nb_seq   : Integer := 2000);
	port(	mic_1,mic_2,son	   : in std_logic_vector(NB_BITS - 1 downto 0); --entre son: son connu
			mu_S       				: in std_logic_vector(NB_BITS - 1 downto 0); --pas de convergence cte
			mu_P       				: in std_logic_vector(NB_BITS - 1 downto 0); --on peut mettre les switch
			Reset,CLK,echantCLK  : in std_logic;
						
			End_modelisation     : in std_logic; -- 1 une fois que le son connu est pass
			
			--SIMU			
			clk_coeff_S_1,clk_coeff_S_2				: out std_logic;
			clk_coeff_P_1,clk_coeff_P_2				: out std_logic;
			coeff_S					: out std_logic_vector(NB_BITS - 1 downto 0); 			
			coeff_P					: out std_logic_vector(NB_BITS - 1 downto 0);
			
			HP1				      : out std_logic_vector(2*NB_BITS - 1 downto 0)); --Sortie HP
end Filtre_FX_LMS ;
    
-- déclaration de l'architecture du testBench
Architecture Filtre_LMS_beh of Filtre_FX_LMS is
      
 -- déclaration des composants

	component Partie_Commande is 
		generic(NB_BITS  : integer ;
				  NB_COEFF : integer;
				  Nb_seq   : integer);
		port(	x,signal_xprim,x_out,wP_out,wS_out,son_out,xprim_out,Calcul_wP_out,Calcul_wS_out,Son_connu	 : in std_logic_vector(NB_BITS - 1 downto 0);
			Reset,CLK,echantCLK										 : in std_logic;
			End_modelisation : in std_logic;
			x_in,wP_in,wS_in,son_in,xprim_in : out std_logic_vector(NB_BITS - 1 downto 0);
			ENw,ENx,ENs,ENson,EN_xprim : out std_logic;   
			EN_sortie, EN_sortie_son : out std_logic;
			ResetMAC_P,ResetMAC_S,ResetMAC_son : out std_logic;
			clk_coeff_S_1,clk_coeff_S_2				: out std_logic;
			clk_coeff_P_1,clk_coeff_P_2				: out std_logic;
			Reset_majS, Reset_majP : out std_logic);
	end component ;
 
	component MAC is
		generic(NB_BITS : integer);
		port ( Entree1,Entree2 : in  std_logic_vector(NB_BITS-1 downto 0);
				  Reset,CLK 	  : in  std_logic;
				  Sortie  		  : out std_logic_vector(2*NB_BITS-1 downto 0));
	end component;

	component Calcul_wP is
		generic(	NB_BITS : integer);
		port(	mic_1,x_prim,w_P	: in std_logic_vector(NB_BITS - 1 downto 0);
				mu_P 	  		: in std_logic_vector(NB_BITS - 1 downto 0);
				Reset 		: in std_logic;
				Sortie		: out std_logic_vector(NB_BITS - 1 downto 0));
	end component;
	
	component Calcul_wS is
		generic(NB_BITS : integer);
		port(	mic_1,signal_est,x,w: in std_logic_vector(NB_BITS - 1 downto 0);
				mu : std_logic_vector(NB_BITS - 1 downto 0);
				Reset : in std_logic;
				Sortie: out std_logic_vector(NB_BITS - 1 downto 0));			
	end component;

	component FIFO is
		generic( NB_BITS : integer;
					NB_COEFF : integer);
		port(	D					: in std_logic_vector(NB_BITS - 1 downto 0);
				Reset,CLK,EN	: in std_logic;
				Q					: out std_logic_vector(NB_BITS - 1 downto 0));		  
	end component;


	component Arrondi is
		generic(	NB_BITS : integer);
		port(	Entree	: in std_logic_vector(2*NB_BITS - 1 downto 0);
				Reset		: in std_logic;
				Sortie	: out std_logic_vector(NB_BITS - 1 downto 0));		
	end component;

	component Bas_D_nB is
		generic( NB_BITS : integer);
		port(	D						: in std_logic_vector(NB_BITS - 1 downto 0);
				Reset,CLK,EN	: in std_logic;
				Q						: out std_logic_vector(NB_BITS - 1 downto 0));
	end component;  


	signal x_in,x_out,wP_in,wP_out			   : std_logic_vector(NB_BITS - 1 downto 0) := (others =>'0');
	signal xprim_in, xprim_out  : std_logic_vector(NB_BITS - 1 downto 0) := (others =>'0');
	signal Calcul_wP_out, Calcul_wS_out, signal_xprim, signal_yS     : std_logic_vector(NB_BITS - 1 downto 0) := (others =>'0');
	signal MACy_out,MACx_out,MACson_out	   : std_logic_vector(2*NB_BITS - 1 downto 0);
	signal signal_esttmp, signal_xprim_tmp, signal_yS_tmp : std_logic_vector(2*NB_BITS - 1 downto 0);
	signal ResetMAC_P,ResetMAC_S,ResetMAC_son   : std_logic;
	signal Reset_majS, Reset_majP : std_logic;
	signal wS_in,wS_out,son_in,son_out : std_logic_vector(NB_BITS - 1 downto 0) := (others =>'0');
	signal ENw, ENx, ENs, ENson, EN_xprim, EN_sortie, EN_sortie_Son, S_calcul : std_logic := '0' ;


	BEGIN 
	          	          		       	  	  	
	FIFOx :FIFO 
	generic map(NB_BITS, NB_COEFF)
	port map(D=>x_in,Reset=>Reset,CLK=>CLK,EN=>ENx,Q=>x_out);

	FIFOp :FIFO 
	generic map(NB_BITS, NB_COEFF)
	port map(D=>wP_in,Reset=>Reset,CLK=>CLK,EN=>ENw,Q=>wP_out);

	FIFOs :FIFO 
	generic map(NB_BITS, NB_COEFF)
	port map(D=>wS_in,Reset=>Reset,CLK=>CLK,EN=>ENs,Q=>wS_out);
	
	FIFOson :FIFO 
	generic map(NB_BITS, NB_COEFF)
	port map(D=>son_in,Reset=>Reset,CLK=>CLK,EN=>ENson,Q=>son_out);
	
	FIFOxprim :FIFO 
	generic map(NB_BITS, NB_COEFF)
	port map(D=>xprim_in,Reset=>Reset,CLK=>CLK,EN=>EN_xprim,Q=>xprim_out);

	MACp : MAC
	generic map(NB_BITS)
	port map (Entree1=>x_out,Entree2=>wP_out,CLK=>CLK, Reset=>ResetMAC_P,Sortie=>MACy_out);

	MACs : MAC
	generic map(NB_BITS)
	port map (Entree1=>x_out,Entree2=>wS_out,CLK=>CLK, Reset=>ResetMAC_S,Sortie=>MACx_out);

	MACson : MAC
	generic map(NB_BITS)
	port map (Entree1=>son_out,Entree2=>wS_out,CLK=>CLK, Reset=>ResetMAC_Son,Sortie=>MACson_out);
	
	Registre_Sortie : Bas_D_nB
	generic map( 2*NB_BITS )
	port map	(D=>MACy_out ,Reset=>Reset,CLK=>CLK,EN=>EN_sortie, Q=>signal_esttmp);
	
--	Registre_Xprim : Bas_D_nB
--	generic map( 2*NB_BITS )
--	port map	(D=>MACx_out ,Reset=>Reset,CLK=>CLK,EN=>EN_sortie, Q=>signal_xprim_tmp);
	
	Registre_son : Bas_D_nB
	generic map( 2*NB_BITS )
	port map	(D=>MACson_out ,Reset=>Reset,CLK=>CLK,EN=>EN_sortie_Son, Q=>signal_yS_tmp);
	
	Arrondi_Xprim : Arrondi
	generic map(NB_BITS)
	port map(Entree=>MACx_out,Reset=>Reset,Sortie=>signal_xprim);
	
	Arrondi_son : Arrondi
	generic map(NB_BITS)
	port map(Entree=>signal_yS_tmp,Reset=>Reset,Sortie=>signal_yS);

	Calcul_w_P : Calcul_wP
	generic map(NB_BITS)
	port map (mic_1=>mic_1,x_prim=>xprim_out,w_P=>wP_out,mu_P=>mu_P,Reset=>Reset_majP,Sortie=>Calcul_wP_out); 

	Calcul_w_S : Calcul_wS
	generic map(NB_BITS)
	port map (mic_1=>mic_1,signal_est=>signal_yS,x=>son_out,w=>wS_out,mu=>mu_S,Reset=>Reset_majS,Sortie=>Calcul_wS_out); 


	PartieCommande : Partie_Commande
	generic map(NB_BITS,
					NB_COEFF,
					Nb_seq)
	port map(x=>mic_2,
				signal_xprim => signal_xprim,
				x_out=>x_out,
				wP_out=>wP_out,
				wS_out => wS_out,
				son_out => son_out,
				xprim_out => xprim_out,
				Calcul_wP_out=>Calcul_wP_out,
				Calcul_wS_out=>Calcul_wS_out,
				Son_connu => son,
				Reset=>Reset,
				CLK=>CLK,
				echantCLK=>echantCLK,
				End_modelisation=>End_modelisation,
				x_in=>x_in,
				wP_in=>wP_in,
				wS_in => wS_in,
				son_in => son_in,
				xprim_in => xprim_in,
				ENw=>ENw, 
				ENx=>ENx,
				ENs=>ENs,
				EN_xprim => EN_xprim,				
				EN_sortie=>EN_sortie,
				EN_sortie_son=>EN_sortie_son,
				ENson=>ENson,
				ResetMAC_P=>ResetMAC_P,
				ResetMAC_S=>ResetMAC_S,
				ResetMAC_Son=>ResetMAC_Son,
				clk_coeff_S_1	=> clk_coeff_S_1,
				clk_coeff_S_2 => clk_coeff_S_2,
				clk_coeff_P_1	=> clk_coeff_P_1,
				clk_coeff_P_2 => clk_coeff_P_2,
				Reset_majS=>Reset_majS,
				Reset_majP=>Reset_majP); 	
		 
	HP1<=signal_esttmp;
	coeff_S <= Calcul_wS_out;
	coeff_P <= Calcul_wP_out;
	
end Filtre_LMS_beh;

