library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_signed.all;  

-- déclaration de l'entité
entity Partie_Commande is 
	generic(NB_BITS : integer := 12;
			 NB_COEFF : integer:= 50;
			 Nb_seq   : integer:=2000);	 
	port(	x,signal_xprim,x_out,wP_out,wS_out,son_out,xprim_out,Calcul_wP_out,Calcul_wS_out,Son_connu	 : in std_logic_vector(NB_BITS - 1 downto 0);
			Reset,CLK,echantCLK										 : in std_logic;
			End_modelisation : in std_logic;
			x_in,wP_in,wS_in,son_in,xprim_in			 : out std_logic_vector(NB_BITS - 1 downto 0);
			ENw,ENx,ENs,ENson,EN_xprim : out std_logic;   
			EN_sortie, EN_sortie_son : out std_logic;
			ResetMAC_P,ResetMAC_S,ResetMAC_son : out std_logic;
			clk_coeff_S_1,	clk_coeff_S_2			: out std_logic;
			clk_coeff_P_1,	clk_coeff_P_2			: out std_logic;
			Reset_majS, Reset_majP : out std_logic);
end Partie_Commande ;
    
-- déclaration de l'architecture du testBench
Architecture Partie_Commande_beh of Partie_Commande is
      

signal loopFIFOw, loopFIFOx, loopFIFOs, loopFIFOson, loopFIFOxprim : std_logic :='1';
signal echantOK : std_logic := '0';


type Def_Etat is (Init_S, echant_S, calcul_Sout, calcul_wS, Init_P, echant_P, calcul_sortie, calcul_wP );

signal ETAT_COURANT : Def_Etat := Init_S;


Begin 

	DetecEchant : process(echantCLK,ETAT_COURANT,Reset)
	Begin
		if Reset ='1' OR  ETAT_COURANT = echant_S OR ETAT_COURANT = echant_P then
			echantOK <= '0';
		elsif rising_edge(echantCLK) then
			if ETAT_COURANT = Init_S OR ETAT_COURANT = Init_P  then -- L'chantillonnage se dbloque sur front montant si l'tat est revenu  l'INIT
				echantOK <= '1';
			else
				echantOK <= '0';
			end if;
		end if;
	end process;


	etat : process (CLK,Reset,echantOK,End_modelisation)
	variable CPTCalcul_S : integer range 0 to NB_COEFF -1 ;
	variable CPTCalcul_P : integer range 0 to NB_COEFF -1 ;

	Begin 
		if Reset ='1' then
			ETAT_COURANT <= Init_S;
			ENx <= '0';
			ENw <= '0';
			ENs <= '0';
			ENson <= '0';
			EN_xprim <= '0';
			loopFIFOw<='1';
			loopFIFOx <='1'; 
			loopFIFOs<='1';
			loopFIFOson <='1';
			loopFIFOxprim <= '1';		
			ResetMAC_P <= '1';
			ResetMAC_S <= '1';
			ResetMAC_Son <= '1';
			Reset_majP <= '1';
			Reset_majS <= '1';
			CPTCalcul_P:= 0;
			CPTCalcul_S:= 0;
			EN_sortie <= '0';
			EN_sortie_son <= '0';
			
		elsif rising_edge(CLK) then

			case ETAT_COURANT is
				
				------------------
				-- Etape d'estimation du canal S
				------------------
				when Init_S =>
				
				-- Echantillonnage du signal d'entre
				if echantOK = '1' then
					if End_modelisation = '1' then
						ETAT_COURANT <= Init_P;
					else
						ETAT_COURANT <= echant_S; 
					end if;
					loopFIFOx <='1';       
					loopFIFOw<='1';			
					loopFIFOs<='1';
					loopFIFOson<='0';
					loopFIFOxprim<='1';
					ENx <= '0';		
					ENw <= '0';
					ENs <= '0';
					ENson <= '1';					
					EN_sortie <= '0';
					EN_sortie_son <= '0';
					EN_xprim <= '0';
					ResetMAC_P <= '1';
					ResetMAC_S <= '1';
					ResetMAC_Son <= '1';
					Reset_majP <= Reset;
					Reset_majS <= Reset;		
					CPTCalcul_S := 0;
												
				else
					
					clk_coeff_S_1 <= '0';
					clk_coeff_S_2 <= '0';
					loopFIFOx <='1'; 
					loopFIFOw<='1';
					loopFIFOs<='1';
					loopFIFOson<='1';
					loopFIFOxprim<='1';
					ENx <= '0';
					ENw <= '0';
					ENs <= '0';
					ENson <= '0';
					EN_xprim <= '0';
					ResetMAC_P <= '1';
					ResetMAC_S <= '1';
					ResetMAC_Son <= '1';
					Reset_majP <= Reset;
					Reset_majS <= Reset;
					CPTCalcul_S := 0;
					EN_sortie <= '0';	
					EN_sortie_son <= '0';
					
				end if;
				
				when  echant_S =>
				
						ETAT_COURANT <= calcul_Sout; 
						loopFIFOx<='1';			
						loopFIFOw<='1';
						loopFIFOs<='1';
						loopFIFOson<='1';
						loopFIFOxprim<='1';
						ENx <= '0';
						ENw <= '0';			
						ENs <= '1';
						ENson <= '1';
						EN_xprim <= '0';
						ResetMAC_P <= '1';
						ResetMAC_S <= '1';
						ResetMAC_Son <= Reset;
						Reset_majP <= Reset;
						Reset_majS <= Reset;  
						clk_coeff_S_1 <= '1';
						CPTCalcul_S:= 0;

				when   calcul_Sout =>
		
					if CPTCalcul_S = NB_COEFF -1 then
				
						ETAT_COURANT <= calcul_wS; 
						clk_coeff_S_1 <= '0';
						clk_coeff_S_2 <= '1';
						EN_sortie <= '0';
						EN_sortie_son <= '0';
						CPTCalcul_S := 0;				
						ENw <= '0';
						ENx <= '0';
						ENs <= '1';
						ENson <= '1';
						EN_xprim <= '0';
						loopFIFOx<='1';
						loopFIFOw<='1';						
						loopFIFOs<='0';
						loopFIFOson<='1';
						loopFIFOxprim<='1';						
						ResetMAC_P <= '1';
						ResetMAC_S <= '1';
						ResetMAC_Son <= Reset;
						Reset_majP <= Reset;
						Reset_majS <= Reset; 						
								
					elsif CPTCalcul_S = NB_COEFF -2 then
					
						ETAT_COURANT <= calcul_Sout; 
						CPTCalcul_S:= CPTCalcul_S+1; 		
						loopFIFOw <='1';				
						loopFIFOx <='1';				
						loopFIFOs<='1';
						loopFIFOson<='1';
						loopFIFOxprim<='1';
						ENw <= '0';				
						ENx <= '0';
						ENs <= '1';
						ENson <= '1';
						EN_sortie <= '0';
						EN_sortie_son <= '1';
						EN_xprim <= '0';
						ResetMAC_P <= '1';
						ResetMAC_S <= '1';
						ResetMAC_Son <= Reset;
						Reset_majP <= Reset;
						Reset_majS <= Reset;
						
					else
						ETAT_COURANT <= calcul_Sout; 
						CPTCalcul_S:= CPTCalcul_S+1; 		
						loopFIFOw <='1';			
						loopFIFOx <='1';			
						loopFIFOs<='1';
						loopFIFOson<='1';
						loopFIFOxprim<='1';
						ENw <= '0';			
						ENx <= '0';
						ENs <= '1';
						ENson <= '1';
						EN_sortie <= '0';
						EN_sortie_son <= '0';
						EN_xprim <= '0';
						ResetMAC_P <= '1';
						ResetMAC_S <= '1';
						ResetMAC_Son <= Reset;
						Reset_majP <= Reset;
						Reset_majS <= Reset;	
						clk_coeff_S_1	<= '1';					
					end if;
					
					when calcul_wS =>
		
					if CPTCalcul_S = NB_COEFF -1 then
							
						ENw <= '0'; 
						ENx <= '0';
						ENs <= '0';
						ENson <= '0';
						EN_xprim <= '0';
						loopFIFOw <='1';
						loopFIFOx <='1';
						loopFIFOs<='1';
						loopFIFOson<='1';
						loopFIFOxprim<='1';
						EN_sortie <= '0';
						EN_sortie_son <= '0';
						CPTCalcul_S:= 0;
						ResetMAC_P <= '1';
						ResetMAC_S <= '1';
						ResetMAC_Son <= '1';
						Reset_majP <= Reset;
						Reset_majS <= Reset;
						clk_coeff_S_2 <= '0';
						ETAT_COURANT <= Init_S;
							
					else 
							
						ETAT_COURANT <= calcul_wS;
						clk_coeff_S_2 <= '1';
						ENw <= '0';
						ENx <= '0';
						ENs <= '1';
						ENson <= '1';
						EN_xprim <= '0';
						loopFIFOx<='1';
						loopFIFOw<='1';
						loopFIFOson<='1';
						loopFIFOs<='0';
						loopFIFOxprim<='1';
						CPTCalcul_S:= CPTCalcul_S+1;
						ResetMAC_P <= '1';
						ResetMAC_S <= '1';
						ResetMAC_Son <= '1';
						Reset_majP <= Reset;
						Reset_majS <= Reset;
						EN_sortie <= '0';
						EN_sortie_son <= '0';
					end if;
						
				------------------
				-- Etat Initial : Attente de l'chantillonage
				------------------
				when Init_P => 
							
				-- Echantillonnage du signal d'entre
				if echantOK = '1' then

					ETAT_COURANT <= echant_P; -- Prochain tat : chantillonnage
					loopFIFOx <='0';        -- Entre de la FIFO des chantillons gal  la valeur  chantilloner
					loopFIFOw<='1';			-- Entre de la FIFO des coefficients gale  la sortie
					loopFIFOs<='1';
					loopFIFOson<='1';
					loopFIFOxprim<='1';
					ENx <= '1';			-- Activation de la FIFO contenant les chantillons
					ENw <= '0';
					ENs <= '0';
					ENson <= '0';
					EN_xprim <= '0';					
					EN_sortie <= '0';
					EN_sortie_son <= '0';
					ResetMAC_P <= '1';
					ResetMAC_S <= '1';
					ResetMAC_Son <= '1';
					Reset_majP <= Reset;
					Reset_majS <= Reset;
					CPTCalcul_P:= 0;
												
				else
					ETAT_COURANT <= Init_P; 
					clk_coeff_P_1 <= '0';
					clk_coeff_P_2 <= '0';
					loopFIFOx <='0'; 
					loopFIFOw<='1';
					loopFIFOs<='1';
					loopFIFOson<='1';
					loopFIFOxprim<='1';
					ENx <= '0';
					ENw <= '0';
					ENs <= '0';
					ENson <= '0';
					EN_xprim <= '0';
					ResetMAC_P <= '1';
					ResetMAC_S <= '1';
					ResetMAC_Son <= '1';
					Reset_majP <= Reset;
					Reset_majS <= Reset;
					CPTCalcul_P := 0;
					EN_sortie <= '0';	
					EN_sortie_son <= '0';
					
				end if;
		
				------------------	
				-- FIN Echantillonnage 
				-- Dbut des calculs de la sortie
				------------------
				when  echant_P =>
				
						ETAT_COURANT <= calcul_sortie; -- Prochain tat : Calcul sortie
						loopFIFOx<='1';			-- Entre de la FIFO des chantillons gale  la sortie
						loopFIFOw<='1';
						loopFIFOs<='1';
						loopFIFOson<='1';
						loopFIFOxprim<='1';
						ENx <= '1';
						ENw <= '1';			-- Activation de la FIFO contenant les coefficients
						ENs <= '1';
						ENson <= '0';
						EN_xprim <= '0';
						ResetMAC_P <= '0';
						ResetMAC_S <= '0';
						ResetMAC_Son <= '1';
						Reset_majP <= Reset;
						Reset_majS <= Reset;
						clk_coeff_P_1 <= '1';
						CPTCalcul_P:= 0;				-- Mise  zro du compteur pour le calcul de la sortie
						
				------------------
				-- Si les calculs sont termins (Les FIFO sont revenues au point de dpart)
				-- alors Dbut des calculs des coefficients et FIN des calculs de la sortie
				------------------
				when   calcul_sortie =>
		
					if CPTCalcul_P = NB_COEFF -1 then
				
						ETAT_COURANT <= calcul_wP; -- Prochain tat : Mise  jour des coefficients
						clk_coeff_P_1 <= '0';
						clk_coeff_P_2 <= '1';
						EN_sortie <= '0';
						EN_sortie_son <= '0';
						CPTCalcul_P := 0;					-- Mise  zro du compteur pour le calcul des coefficients	
						ENw <= '1';
						ENx <= '1';
						ENs <= '1';
						ENson <= '0';
						EN_xprim <= '1';
						loopFIFOx<='1';
						loopFIFOs<='1';
						loopFIFOson<='1';						
						loopFIFOw<='0';					-- Entre de la FIFO des coefficients diffrente de la sortie
						loopFIFOxprim<='1';
						ResetMAC_P <= Reset;
						ResetMAC_S <= Reset;
						ResetMAC_Son <= '1';
						Reset_majP <= Reset;
						Reset_majS <= Reset;
															
						
								
					elsif CPTCalcul_P = NB_COEFF -2 then
					
						ETAT_COURANT <= calcul_sortie; 
						CPTCalcul_P:= CPTCalcul_P+1; 		
						loopFIFOw <='1';					-- Les FIFOs tournent normalement pour fournir
						loopFIFOx <='1';					-- les valeurs des chantillons et des coefficients
						loopFIFOs<='1';
						loopFIFOson<='1';
						loopFIFOxprim<='0';
						ENw <= '1';					--  l'unit de calcul
						ENx <= '1';
						ENs <= '1';
						ENson <= '0';
						EN_xprim <= '1';
						EN_sortie <= '1';-- Rcupration de la valeur du signal de sortie
						EN_sortie_son <= '0';
						ResetMAC_P <= Reset;
						ResetMAC_S <= Reset;
						ResetMAC_Son <= '1';
						Reset_majP <= Reset;
						Reset_majS <= Reset;
						clk_coeff_P_1 <= '1';
						
					else
						ETAT_COURANT <= calcul_sortie; 
						CPTCalcul_P := CPTCalcul_P+1; 		
						loopFIFOw <='1';					-- Les FIFOs tournent normalement pour fournir
						loopFIFOx <='1';					-- les valeurs des chantillons et des coefficients
						loopFIFOs<='1';
						loopFIFOson<='1';
						loopFIFOxprim<='1';
						ENw <= '1';					--  l'unit de calcul
						ENx <= '1';
						ENs <= '1';
						ENson <= '0';
						EN_xprim <= '0';
						EN_sortie <= '0';-- Rcupration de la valeur du signal de sortie
						EN_sortie_son <= '0';
						ResetMAC_P <= Reset;
						ResetMAC_S <= Reset;
						ResetMAC_Son <= '1';
						Reset_majP <= Reset;
						Reset_majS <= Reset;	
						clk_coeff_P_1	<='1';					
					end if;
	
	
				when calcul_wP =>
		
					------------------
					-- Si MAJ des coefficients termine, alors retour  l'tat initial
					------------------
					if CPTCalcul_P = NB_COEFF -1 then
							
						ENw <= '0'; -- "gel" des FIFOs
						ENx <= '0';
						ENs <= '0';
						ENson <= '0';
						EN_xprim <= '0';
						loopFIFOw <='1';
						loopFIFOx <='1';
						loopFIFOs<='1';
						loopFIFOson<='1';
						loopFIFOxprim<='1';
						EN_sortie <= '0';
						EN_sortie_son <= '0';
						CPTCalcul_P:= 0;
						ResetMAC_P <= '1';
						ResetMAC_S <= '1';
						ResetMAC_Son <= '1';
						Reset_majP <= Reset;
						Reset_majS <= Reset;	
						clk_coeff_P_2 <= '0';
						ETAT_COURANT <= Init_P;
							
					else 
							
						ETAT_COURANT <= calcul_wP;
						ENw <= '1';
						ENx <= '1';
						ENs <= '1';
						ENson <= '0';
						EN_xprim <= '1';
						loopFIFOx<='1';
						loopFIFOw<='0';
						loopFIFOs<='1';
						loopFIFOson<='1';
						loopFIFOxprim<='1';
						CPTCalcul_P := CPTCalcul_P+1;
						ResetMAC_P <= '1';
						ResetMAC_S <= '1';
						ResetMAC_Son <= '1';
						Reset_majP <= Reset;
						Reset_majS <= Reset;	
						EN_sortie <= '0';	
						EN_sortie_son <= '0';
						clk_coeff_P_2 <= '1';
					end if;
		
				when others => 
					
					ETAT_COURANT <= Init_P;
					CPTCalcul_P:= 0; 
					CPTCalcul_S:= 0; 
					ENx <= '0';
					ENw <= '0';
					ENs <= '0';
					ENson <= '0';
					EN_xprim <= '0';
					loopFIFOw<='1';
					loopFIFOx <='1'; 
					loopFIFOs<='1';
					loopFIFOson<='1';
					loopFIFOxprim<='1';
					ResetMAC_P <= '1';
					ResetMAC_S <= '1';
					ResetMAC_Son <= '1';
					Reset_majP <= Reset;
					Reset_majS <= Reset;
					EN_sortie <= '0';
					EN_sortie_son <= '0';
			end CASE;
		end if;
	end process;



	rebouclageFIFO : process (wP_out,wS_out,son_out,x_out,xprim_out,x,signal_xprim,loopFIFOx,loopFIFOw,loopFIFOs,loopFIFOson,loopFIFOxprim,Calcul_wP_out,Calcul_wS_out,Son_connu)
	begin 
		if loopFIFOw='1' then   -- Rebouclage sortie FIFO sur l'entre
			wP_in <= wP_out;
		else 
			wP_in <= Calcul_wP_out; -- MAJ des coefficients sinon
		end if;

		if loopFIFOx='1' then  -- Rebouclage sortie FIFO sur l'entre
			x_in <= x_out;
		else
			x_in<= x;				-- Echantillonnage sinon
		end if;
		
		if loopFIFOs='1' then  -- Rebouclage sortie FIFO sur l'entre
			wS_in <= wS_out;
		else
			wS_in<= Calcul_wS_out;				-- Remplissage sinon
		end if;
		
		if loopFIFOson='1' then  -- Rebouclage sortie FIFO sur l'entre
			son_in <= son_out;
		else
			son_in<= Son_connu;				-- Remplissage sinon
		end if;
		
		if loopFIFOxprim = '1' then
			xprim_in <= xprim_out;
		else
			xprim_in <= signal_xprim;
		end if;
	end process;
			


end Partie_Commande_beh;

