library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity Clock1Hz is
port (
		clk 	: in STD_LOGIC;
		reset : in STD_LOGIC;
		en 	: in STD_LOGIC;	
		div	: in STD_LOGIC_VECTOR(27 downto 0); --Rpz la division de la clock, pour 1Hz, div = 5F5E100 (=100M)
		clk_out: out STD_LOGIC
	);
end Clock1Hz;

architecture Behavioral of Clock1Hz is

--Component--
Component ADD is
    generic(width : integer := 28);
	port (A, B  : in  STD_LOGIC_VECTOR(width-1 downto 0); -- A + B et Cout
		  S     : out STD_LOGIC_VECTOR(width-1 downto 0) --Sortie et Carrie
	  );	
end Component;

--Le Registre Reg28bits se remet a zero un fois la valeur 100M atteinte, ce qui correspond a une seconde

Component Reg
        generic(width : integer := 28);
		Port (  clk   : in STD_LOGIC;
				RESET : in  STD_LOGIC; -- Bouton rouge
				EN    : in STD_LOGIC; --BTNC	
				div	: in STD_LOGIC_VECTOR(width-1 downto 0); --100M = 5F5E100 soit 1 seconde 2faf080 pour 0.5 (plus rapide)
				D		: in STD_LOGIC_VECTOR(width-1 downto 0);
				Q     : out STD_LOGIC_VECTOR(width-1 downto 0));
end Component;
 
--signal--
signal RegOut : STD_LOGIC_VECTOR(27 downto 0);
signal RegIn  : STD_LOGIC_VECTOR(27 downto 0);
signal Bin : STD_LOGIC_VECTOR(27 downto 0);

signal clkout_sig : STD_LOGIC;

begin
	ADD0 : ADD generic map(28)
	       port map(A => RegOut ,B => Bin ,S => RegIn);
	Reg0 : Reg generic map(28)  port map(clk => clk,reset => reset ,en => en , div => div, D => RegIn, Q => RegOut);

process(clk, reset, RegIn)
begin

	if(reset = '0')then
		Bin <= (others => '0');
		clk_out <= '0';
		clkout_sig <= '0';		
	elsif(clk'event and clk = '1') then	
		if(EN = '1') then
			Bin <= x"0000001";					
		else
			Bin <= (others => '0');
		end if;			
		if(RegIn = div)then --100M = 5F5E100 soit 1 seconde (1Hz) 2faf080 pour 0.5 (plus rapide)
				clkout_sig <= not clkout_sig;		
		end if;
		clk_out <= clkout_sig;		
	end if;
	
end process;

end Behavioral;