library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity PMOD_DAC is
	GENERIC(d_width : INTEGER := 32); --data bus width
	port(
			reset   : IN STD_LOGIC;
			clock   : IN STD_LOGIC;
			Data	: IN STD_LOGIC_VECTOR(11 downto 0);
			
			mosi    : OUT STD_LOGIC; 
			sclk    : OUT STD_LOGIC;
			ss_n    : BUFFER STD_LOGIC   --slave select
		);
end PMOD_DAC;

architecture Behavioral of PMOD_DAC is

Component spi_master is
	GENERIC(d_width : INTEGER := 32); --data bus width
	port(
			clock   : IN     STD_LOGIC;                             --system clock
			reset_n : IN     STD_LOGIC;                             --asynchronous reset
			enable  : IN     STD_LOGIC;                             --initiate transaction
			cont    : IN     STD_LOGIC;                             --continuous mode command
			tx_data : IN     STD_LOGIC_VECTOR(d_width-1 DOWNTO 0);  --data to transmit
			miso    : IN     STD_LOGIC;                             --master in, slave out
			sclk    : OUT    STD_LOGIC;                             --spi clock
			ss_n    : BUFFER STD_LOGIC;   --slave select
			mosi    : OUT    STD_LOGIC;                             --master out, slave in
			busy    : OUT    STD_LOGIC;                             --busy / data ready signal
			rx_data : OUT    STD_LOGIC_VECTOR(d_width-1 DOWNTO 0)   --data received
		);

end component;

signal reset_n: std_logic;
signal cont: std_logic;
signal busy: std_logic;
signal enable_spi: std_logic;
signal tx_data: STD_LOGIC_VECTOR(d_width-1 DOWNTO 0);

TYPE machine IS(clear_0, clear_1, set_ref_0,set_ref_1, set_power_0,set_power_1, send_0,send_1);
SIGNAL state       : machine; 

SIGNAL count       : INTEGER;    

begin

	SPI : spi_master 
		generic map(d_width) --data bus width
		port map (
		  clock => clock,
          reset_n => reset_n,
          enable => enable_spi,
          cont => cont,
          tx_data => tx_data, --Data  transmettre
          miso => '0',
          sclk => sclk,
          ss_n => ss_n,
          mosi => mosi,
          busy => busy,
          rx_data => open	
		);

	process(clock,reset)
	begin
		
		if(reset = '1') then
			reset_n <= '0';			
			enable_spi <= '0';
			tx_data <= (others => '0');
			state <= clear_0;  			
			
		elsif(clock'event and clock = '1')then			
			case(state)is
			
			WHEN clear_0 => --Optionnel
			     reset_n <= '1';	
			     cont <= '0';
			     enable_spi <= '1';			     
			     tx_data <= "00000101000000000000000000000010";
			     if(enable_spi = '1')then
                    state <= clear_1;                                    
                end if;                
           WHEN clear_1 =>
                enable_spi <= '0';
                if(busy = '0') then
                    state <= set_ref_0;
                end if;		
                	 
	   --SET REF ON
			WHEN set_ref_0 => --Envoie la commande xxxx 1000 xxxx xxxx xxxx xxxx xxxx xxx1 pour set la rfrence  ON				
				--reset_n <= '1';
				enable_spi <= '1'; 
				--cont <= '0';									
				tx_data <= 	"00001000000000000000000000000001";								
			     if(enable_spi = '1')then
				    state <= set_ref_1;									
			     end if;
			WHEN set_ref_1 =>			
				enable_spi <= '0';
				if(busy = '0') then
					state <= set_power_0;
				end if;
				
	   --SET POWER UP
			WHEN set_power_0 => --Envoie:   xxxx 0011 1111 xxxx xxxx xx00 0000 0000 mode normal et power-up							
				enable_spi <= '1';
				tx_data <= "00000011111100000000000000000000";					
				if(enable_spi = '1')then
					state <= set_power_1;
				end if;				
			WHEN set_power_1 =>			
				enable_spi <= '0';				
				if(busy = '0') then
					state <= send_0;					
				end if;
				
		--SEND
			WHEN send_0 => --Envoie donnes: xxxx 0011 1111 Data 0000 0000 vers tout les DACs
				enable_spi <= '1';
				tx_data <= "000000111111" & Data & "00000000";				
				if(enable_spi = '1')then
					state <= send_1;
				end if;				
			WHEN send_1 =>
				enable_spi <= '0';				
				if(busy = '0') then
					enable_spi <= '0';
					state <= send_0;					
				end if;
		    
		    WHEN others =>
		          state <= set_ref_0;
			
			end case;
		end if;		
	end process;
	
	
	

end Behavioral;

