library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity top_level is
    Port (  clk : in STD_LOGIC;
            reset : in STD_LOGIC;

            sw : in STD_LOGIC_VECTOR(11 downto 0);
            
            --Ports ADC
            vauxp6 : in STD_LOGIC;
            vauxn6 : in STD_LOGIC;
            vauxp14 : in STD_LOGIC;
            vauxn14 : in STD_LOGIC;
       
            --Ports DAC
            mosi : out std_logic; 
            sclk : out std_logic;
            ss_n : buffer std_logic
       );
end top_level;

architecture Behavioral of top_level is

--Composant pour la mmoire:
Component blk_mem_gen_0 IS
  PORT (
    clka : IN STD_LOGIC;
    wea : IN STD_LOGIC_VECTOR(0 DOWNTO 0);
    addra : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
    dina : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
    douta : OUT STD_LOGIC_VECTOR(15 DOWNTO 0)
  );
END Component;

component Clock1Hz is
port (
		clk 	: in STD_LOGIC;
		reset : in STD_LOGIC;
		en 	: in STD_LOGIC;	
		div	: in STD_LOGIC_VECTOR(27 downto 0); --Rpz la division de la clock, pour 1Hz, div = 5F5E100 (=100M)
		clk_out: out STD_LOGIC
	);
end component;

component Compteur is
generic (width : integer := 16);
port(
		clk : in STD_LOGIC;
		en : in std_logic;
		cptOut 	: out  STD_LOGIC_VECTOR (width-1 downto 0)
	);
end component;

--Composants ADC
component xadc_wiz_0 is
   port
    (
    daddr_in        : in  STD_LOGIC_VECTOR (6 downto 0);     -- Address bus for the dynamic reconfiguration port
    den_in          : in  STD_LOGIC;                         -- Enable Signal for the dynamic reconfiguration port
    di_in           : in  STD_LOGIC_VECTOR (15 downto 0);    -- Input data bus for the dynamic reconfiguration port
    dwe_in          : in  STD_LOGIC;                         -- Write Enable for the dynamic reconfiguration port
    do_out          : out  STD_LOGIC_VECTOR (15 downto 0);   -- Output data bus for dynamic reconfiguration port / UNIQUEMENT les 12 bits MSB sont significatif
    drdy_out        : out  STD_LOGIC;                        -- Data ready signal for the dynamic reconfiguration port
    dclk_in         : in  STD_LOGIC;                         -- Clock input for the dynamic reconfiguration port    
    vauxp6          : in  STD_LOGIC;                         -- Auxiliary Channel 6
    vauxn6          : in  STD_LOGIC;
    vauxp14         : in  STD_LOGIC;                         -- Auxiliary Channel 14
    vauxn14         : in  STD_LOGIC;
    busy_out        : out  STD_LOGIC;                        -- ADC Busy signal
    channel_out     : out  STD_LOGIC_VECTOR (4 downto 0);    -- Channel Selection Outputs
    eoc_out         : out  STD_LOGIC;                        -- End of Conversion Signal
    eos_out         : out  STD_LOGIC;                        -- End of Sequence Signal
    alarm_out       : out STD_LOGIC;                         -- OR'ed output of all the Alarms
    vp_in           : in  STD_LOGIC;                         -- Dedicated Analog Input Pair
    vn_in           : in  STD_LOGIC
);
end component;

--Troncateur 12-16

Component Troncateur_12unsigned_16signed is
    Port ( In_b : in STD_LOGIC_VECTOR (11 downto 0);
       Out_b : out STD_LOGIC_VECTOR (11 downto 0));
end Component;

--Composant FIR

Component Filtre_FX_LMS is 
	generic(NB_BITS  : integer :=16 ;
			  NB_COEFF : integer :=50 ;
			  Nb_seq   : Integer := 2000);
	port(	mic_1,mic_2,son	   : in std_logic_vector(NB_BITS - 1 downto 0); --entre son: son connu
			mu_S       				: in std_logic_vector(NB_BITS - 1 downto 0); --pas de convergence cte
			mu_P       				: in std_logic_vector(NB_BITS - 1 downto 0); --on peut mettre les switch
			Reset,CLK,echantCLK  : in std_logic;
						
			End_modelisation     : in std_logic; -- 1 une fois que le son connu est pass
			
			--SIMU			
			clk_coeff_S_1,clk_coeff_S_2				: out std_logic;
			clk_coeff_P_1,clk_coeff_P_2				: out std_logic;
			coeff_S					: out std_logic_vector(NB_BITS - 1 downto 0); 			
			coeff_P					: out std_logic_vector(NB_BITS - 1 downto 0);
			
			HP1				      : out std_logic_vector(2*NB_BITS - 1 downto 0)); --Sortie HP
end Component;

--Troncateur 32-12

Component Troncateur_32_12 is
generic(NB_BITS : integer := 32);
port(	In_b: in std_logic_vector(NB_BITS - 1 downto 0);
	   Out_b: out std_logic_vector(11 downto 0));	   
end Component;

component Troncateur_16_12 is
generic(NB_BITS : integer := 16);
port(	In_b: in std_logic_vector(NB_BITS - 1 downto 0);
	   Out_b: out std_logic_vector(11 downto 0));	   
end component;

--Composant DAC
component PMOD_DAC is
GENERIC(d_width : INTEGER := 32); --data bus width
port(
        reset   : IN STD_LOGIC;
        clock   : IN STD_LOGIC;
        Data    : IN STD_LOGIC_VECTOR(11 downto 0);
        
        mosi    : OUT STD_LOGIC; 
        sclk    : OUT STD_LOGIC;
        ss_n    : BUFFER STD_LOGIC   --slave select
    );
end component;  


--Signaux BRAM

signal cpt : std_logic_vector(15 downto 0); 
signal compteur_son : integer range 0 to 7 := 0;
signal enable_cpt : std_logic;
signal enable_clk_mem : std_logic;

--Signaux ADC

signal data_in_ADC : STD_LOGIC_VECTOR(15 downto 0);
signal enable_ADC : STD_LOGIC; 
signal adrs_ADC_in: STD_LOGIC_VECTOR (6 downto 0) := "0010110";
signal data_ADC : STD_LOGIC_VECTOR (15 downto 0); --Entre dans troncateur 12-16

--Signaux FIR

signal mic_1,mic_2 : STD_LOGIC_VECTOR(11 downto 0);
signal son : STD_LOGIC_VECTOR(15 downto 0);
--signal clka : std_logic;
signal endMod : std_logic;
signal HP1 : std_logic_vector(23 downto 0);
signal echantCLK : std_logic;

--Signaux DAC

signal in_DAC : std_logic_vector(11 downto 0);
signal in_DAC_FIR : std_logic_vector(11 downto 0);

--Signal Troncateur 12-16 -> DAC

signal Data_in_FIR : std_logic_vector(11 downto 0);

--Signal Son out

signal son_out : std_logic_vector(11 downto 0);

begin

    RAM0 : blk_mem_gen_0 port map(
            clka => echantCLK,
            wea  => "0",
            addra => cpt,
            dina => (others => '0'),
            douta => son
        );
        
     clk_48Hz1 : Clock1Hz port map(
                    clk => clk,
                    reset => '1',
                    en => '1',
                    div => x"0000824",--x"824" pour avoir une clock  48kHz
                    clk_out => echantCLK        
                );    
    Cpt0 : Compteur generic map (16) 
           port map(
                    clk => echantCLK,
                    en => enable_cpt,
                    cptOut => cpt);
                    
    process(cpt)
    begin      
        if(cpt >= x"FFF0") then
            enable_cpt <= '0';
            endMod <= '1'; 
            in_DAC <= in_DAC_FIR;   
        else
            enable_cpt <= '1';
            endMod <= '0'; 
            in_DAC <= son_out;
        end if;
    end process;    


--ADC
    XADC0 : xadc_wiz_0 port map(
        daddr_in    => adrs_ADC_in,        
        den_in     =>  enable_ADC,
        di_in => data_in_ADC, 
        dwe_in => '0',
        do_out    => data_ADC,
        drdy_out => open,
        dclk_in => clk,        
        vauxp6     => vauxp6,
        vauxn6     => vauxn6,
        vauxp14     => vauxp14,
        vauxn14     => vauxn14,
        busy_out => open,
        eoc_out     =>   enable_ADC,                  -- End of Conversion Signal
        eos_out     =>  open,        -- End of Sequence Signal
        channel_out => open,        
        alarm_out       => open,                       -- OR'ed output of all the Alarms
        vp_in           => '0',                   -- Dedicated Analog Input Pair
        vn_in           => '0'
        );

--DEMUX
    process(adrs_ADC_in)
    begin
     if(reset = '1')then
         adrs_ADC_in <= "0010110";
     elsif(clk'event and clk = '1')then
        if (adrs_ADC_in = "0010110")then --Vaux 6
            mic_1 <= Data_in_FIR; 
            adrs_ADC_in <= "0011110";           
        elsif(adrs_ADC_in = "0011110")then --Vaux 14
            mic_2 <= Data_in_FIR;
            adrs_ADC_in <= "0010110";
        else
            mic_1 <= (others => '0');
            mic_2 <= (others => '0');
        end if;  
      end if;
    end process;
                
--Troncateur 12-16
troncateur0 : Troncateur_12unsigned_16signed
      port map (In_b => data_ADC(15 downto 4),
             Out_b =>  Data_in_FIR
             );           
             
--FILTRE
Filtre0 : Filtre_FX_LMS
    generic map(NB_BITS => 12 ,
			  NB_COEFF => 50,
			  Nb_seq   => 2000)
	port map(	
	        mic_1 => mic_1 ,
	        mic_2 => mic_2 ,
	        son => son(15 downto 4),	  --entre son: son connu
			mu_S  => "000000100000", --pas de convergence cte
			mu_P  => sw,--on peut mettre les switch pour modifier manuellement
			Reset => Reset,
			CLK => 	clk,
			echantCLK  => echantCLK, --clk  48kHz
						
			End_modelisation => endMod,-- 1 une fois que le son connu est pass
			
			--SIMU			
			clk_coeff_S_1  => open,
			clk_coeff_S_2  => open,
			clk_coeff_P_1  => open,
			clk_coeff_P_2  => open,
            coeff_S        => open,
			coeff_P		   => open,
			
			HP1			   => HP1); --Sortie HP                          

--Troncateur 24-12

troncateur1 : Troncateur_32_12
      generic map(24)
      port map (In_b => HP1,
             Out_b =>  in_DAC_FIR
             );   
             
troncateur2 : Troncateur_16_12
                   generic map(16)
                   port map (In_b => son,
                          Out_b =>  son_out
                          );                         
--DAC        
   DAC0 : PMOD_DAC generic map(32) --data bus width
        port map(
                reset => reset,
                clock => clk,
                Data  => '0' & in_DAC(11 downto 1), 
                
                mosi    => mosi,
                sclk    => sclk,
                ss_n    => ss_n  --slave select
            );    
end Behavioral;
